@@ -4,7 +4,6 @@
 
 use super::{CodeType, TypeIdentifier};
 use crate::interface::FFIType;
-use std::fmt;
 
 /// An object to look up a foreign language code specific renderer for a given type used.
 /// Every `Type` referred to in the `ComponentInterface` should map to a corresponding
@@ -18,19 +17,19 @@ pub trait CodeOracle {
     fn find(&self, type_: &TypeIdentifier) -> Box<dyn CodeType>;
 
     /// Get the idiomatic rendering of a class name (for enums, records, errors, etc).
-    fn class_name(&self, nm: &dyn fmt::Display) -> String;
+    fn class_name(&self, nm: &str) -> String;
 
     /// Get the idiomatic rendering of a function name.
-    fn fn_name(&self, nm: &dyn fmt::Display) -> String;
+    fn fn_name(&self, nm: &str) -> String;
 
     /// Get the idiomatic rendering of a variable name.
-    fn var_name(&self, nm: &dyn fmt::Display) -> String;
+    fn var_name(&self, nm: &str) -> String;
 
     /// Get the idiomatic rendering of an individual enum variant.
-    fn enum_variant_name(&self, nm: &dyn fmt::Display) -> String;
+    fn enum_variant_name(&self, nm: &str) -> String;
 
     /// Get the idiomatic rendering of an error name.
-    fn error_name(&self, nm: &dyn fmt::Display) -> String;
+    fn error_name(&self, nm: &str) -> String;
 
     fn ffi_type_label(&self, ffi_type: &FFIType) -> String;
 }
