@@ -2,15 +2,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-use std::collections::{HashMap, HashSet};
-use std::fmt;
+use std::borrow::Borrow;
+use std::cell::RefCell;
+use std::collections::{BTreeSet, HashMap, HashSet};
 
 use anyhow::Result;
 use askama::Template;
-use heck::{CamelCase, MixedCase, ShoutySnakeCase};
+use heck::{ToLowerCamelCase, ToShoutySnakeCase, ToUpperCamelCase};
 use serde::{Deserialize, Serialize};
 
-use crate::backend::{CodeDeclaration, CodeOracle, CodeType, TemplateExpression, TypeIdentifier};
+use crate::backend::{CodeOracle, CodeType, TemplateExpression, TypeIdentifier};
 use crate::interface::*;
 use crate::MergeWith;
 
@@ -19,7 +20,7 @@ mod compounds;
 mod custom;
 mod enum_;
 mod error;
-mod function;
+mod external;
 mod miscellany;
 mod object;
 mod primitives;
@@ -32,6 +33,8 @@ pub struct Config {
     cdylib_name: Option<String>,
     #[serde(default)]
     custom_types: HashMap<String, CustomTypeConfig>,
+    #[serde(default)]
+    external_packages: HashMap<String, String>,
 }
 
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
@@ -66,6 +69,7 @@ impl From<&ComponentInterface> for Config {
             package_name: Some(format!("uniffi.{}", ci.namespace())),
             cdylib_name: Some(format!("uniffi_{}", ci.namespace())),
             custom_types: HashMap::new(),
+            external_packages: HashMap::new(),
         }
     }
 }
@@ -76,134 +80,122 @@ impl MergeWith for Config {
             package_name: self.package_name.merge_with(&other.package_name),
             cdylib_name: self.cdylib_name.merge_with(&other.cdylib_name),
             custom_types: self.custom_types.merge_with(&other.custom_types),
+            external_packages: self.external_packages.merge_with(&other.external_packages),
         }
     }
 }
 
 // Generate kotlin bindings for the given ComponentInterface, as a string.
 pub fn generate_bindings(config: &Config, ci: &ComponentInterface) -> Result<String> {
-    let oracle = KotlinCodeOracle::new(config.clone());
-    filters::set_oracle(oracle.clone());
-
-    KotlinWrapper::new(oracle, config.clone(), ci)
+    KotlinWrapper::new(config.clone(), ci)
         .render()
         .map_err(|_| anyhow::anyhow!("failed to render kotlin bindings"))
 }
 
+/// Renders Kotlin helper code for all types
+///
+/// This template is a bit different than others in that it stores internal state from the render
+/// process.  Make sure to only call `render()` once.
+#[derive(Template)]
+#[template(syntax = "kt", escape = "none", path = "Types.kt")]
+pub struct TypeRenderer<'a> {
+    kotlin_config: &'a Config,
+    ci: &'a ComponentInterface,
+    // Track included modules for the `include_once()` macro
+    include_once_names: RefCell<HashSet<String>>,
+    // Track imports added with the `add_import()` macro
+    imports: RefCell<BTreeSet<String>>,
+}
+
+impl<'a> TypeRenderer<'a> {
+    fn new(kotlin_config: &'a Config, ci: &'a ComponentInterface) -> Self {
+        Self {
+            kotlin_config,
+            ci,
+            include_once_names: RefCell::new(HashSet::new()),
+            imports: RefCell::new(BTreeSet::new()),
+        }
+    }
+
+    // Get the package name for an external type
+    fn external_type_package_name(&self, crate_name: &str) -> String {
+        match self.kotlin_config.external_packages.get(crate_name) {
+            Some(name) => name.clone(),
+            None => crate_name.to_string(),
+        }
+    }
+
+    // The following methods are used by the `Types.kt` macros.
+
+    // Helper for the including a template, but only once.
+    //
+    // The first time this is called with a name it will return true, indicating that we should
+    // include the template.  Subsequent calls will return false.
+    fn include_once_check(&self, name: &str) -> bool {
+        self.include_once_names
+            .borrow_mut()
+            .insert(name.to_string())
+    }
+
+    // Helper to add an import statement
+    //
+    // Call this inside your template to cause an import statement to be added at the top of the
+    // file.  Imports will be sorted and de-deuped.
+    //
+    // Returns an empty string so that it can be used inside an askama `{{ }}` block.
+    fn add_import(&self, name: &str) -> &str {
+        self.imports.borrow_mut().insert(name.to_owned());
+        ""
+    }
+}
+
 #[derive(Template)]
 #[template(syntax = "kt", escape = "none", path = "wrapper.kt")]
 pub struct KotlinWrapper<'a> {
-    oracle: KotlinCodeOracle,
     config: Config,
     ci: &'a ComponentInterface,
+    type_helper_code: String,
+    type_imports: BTreeSet<String>,
 }
+
 impl<'a> KotlinWrapper<'a> {
-    pub fn new(oracle: KotlinCodeOracle, config: Config, ci: &'a ComponentInterface) -> Self {
-        Self { oracle, config, ci }
+    pub fn new(config: Config, ci: &'a ComponentInterface) -> Self {
+        let type_renderer = TypeRenderer::new(&config, ci);
+        let type_helper_code = type_renderer.render().unwrap();
+        let type_imports = type_renderer.imports.into_inner();
+        Self {
+            config,
+            ci,
+            type_helper_code,
+            type_imports,
+        }
     }
 
-    pub fn members(&self) -> Vec<Box<dyn CodeDeclaration + 'a>> {
-        let ci = self.ci;
-        vec![
-            Box::new(object::KotlinObjectRuntime::new(ci)) as Box<dyn CodeDeclaration>,
-            Box::new(callback_interface::KotlinCallbackInterfaceRuntime::new(ci))
-                as Box<dyn CodeDeclaration>,
-        ]
-        .into_iter()
-        .chain(
-            ci.iter_enum_definitions().into_iter().map(|inner| {
-                Box::new(enum_::KotlinEnum::new(inner, ci)) as Box<dyn CodeDeclaration>
-            }),
-        )
-        .chain(ci.iter_function_definitions().into_iter().map(|inner| {
-            Box::new(function::KotlinFunction::new(inner, ci)) as Box<dyn CodeDeclaration>
-        }))
-        .chain(ci.iter_object_definitions().into_iter().map(|inner| {
-            Box::new(object::KotlinObject::new(inner, ci)) as Box<dyn CodeDeclaration>
-        }))
-        .chain(ci.iter_record_definitions().into_iter().map(|inner| {
-            Box::new(record::KotlinRecord::new(inner, ci)) as Box<dyn CodeDeclaration>
-        }))
-        .chain(
-            ci.iter_error_definitions().into_iter().map(|inner| {
-                Box::new(error::KotlinError::new(inner, ci)) as Box<dyn CodeDeclaration>
-            }),
-        )
-        .chain(
-            ci.iter_callback_interface_definitions()
-                .into_iter()
-                .map(|inner| {
-                    Box::new(callback_interface::KotlinCallbackInterface::new(inner, ci))
-                        as Box<dyn CodeDeclaration>
-                }),
-        )
-        .chain(ci.iter_custom_types().into_iter().map(|(name, type_)| {
-            let config = self.config.custom_types.get(&name).cloned();
-            Box::new(custom::KotlinCustomType::new(name, type_, config)) as Box<dyn CodeDeclaration>
-        }))
-        .collect()
-    }
-
-    pub fn initialization_code(&self) -> Vec<String> {
-        let oracle = &self.oracle;
-        self.members()
+    pub fn initialization_fns(&self) -> Vec<String> {
+        self.ci
+            .iter_types()
             .into_iter()
-            .filter_map(|member| member.initialization_code(oracle))
-            .collect()
-    }
-
-    pub fn declaration_code(&self) -> Vec<String> {
-        let oracle = &self.oracle;
-        self.members()
-            .into_iter()
-            .filter_map(|member| member.definition_code(oracle))
-            .chain(
-                self.ci
-                    .iter_types()
-                    .into_iter()
-                    .filter_map(|type_| oracle.find(&type_).helper_code(oracle)),
-            )
+            .filter_map(|t| t.initialization_fn(&KotlinCodeOracle))
             .collect()
     }
 
     pub fn imports(&self) -> Vec<String> {
-        let oracle = &self.oracle;
-        let mut imports: Vec<String> = self
-            .members()
-            .into_iter()
-            .filter_map(|member| member.imports(oracle))
-            .flatten()
-            .chain(
-                self.ci
-                    .iter_types()
-                    .into_iter()
-                    .filter_map(|type_| oracle.find(&type_).imports(oracle))
-                    .flatten(),
-            )
-            .collect::<HashSet<String>>()
-            .into_iter()
-            .collect();
-
-        imports.sort();
-        imports
+        self.type_imports.iter().cloned().collect()
     }
 }
 
 #[derive(Clone)]
-pub struct KotlinCodeOracle {
-    config: Config,
-}
+pub struct KotlinCodeOracle;
 
 impl KotlinCodeOracle {
-    fn new(config: Config) -> Self {
-        Self { config }
-    }
-
+    // Map `Type` instances to a `Box<dyn CodeType>` for that type.
+    //
+    // There is a companion match in `templates/Types.kt` which performs a similar function for the
+    // template code.
+    //
+    //   - When adding additional types here, make sure to also add a match arm to the `Types.kt` template.
+    //   - To keep things managable, let's try to limit ourselves to these 2 mega-matches
     fn create_code_type(&self, type_: TypeIdentifier) -> Box<dyn CodeType> {
-        // I really want access to the ComponentInterface here so I can look up the interface::{Enum, Record, Error, Object, etc}
-        // However, there's some violence and gore I need to do to (temporarily) make the oracle usable from filters.
-
-        // Some refactor of the templates is needed to make progress here: I think most of the filter functions need to take an &dyn CodeOracle
         match type_ {
             Type::UInt8 => Box::new(primitives::UInt8CodeType),
             Type::Int8 => Box::new(primitives::Int8CodeType),
@@ -228,28 +220,11 @@ impl KotlinCodeOracle {
             Type::CallbackInterface(id) => {
                 Box::new(callback_interface::CallbackInterfaceCodeType::new(id))
             }
-
-            Type::Optional(ref inner) => {
-                let outer = type_.clone();
-                let inner = *inner.to_owned();
-                Box::new(compounds::OptionalCodeType::new(inner, outer))
-            }
-            Type::Sequence(ref inner) => {
-                let outer = type_.clone();
-                let inner = *inner.to_owned();
-                Box::new(compounds::SequenceCodeType::new(inner, outer))
-            }
-            Type::Map(ref inner) => {
-                let outer = type_.clone();
-                let inner = *inner.to_owned();
-                Box::new(compounds::MapCodeType::new(inner, outer))
-            }
-            Type::External { .. } => panic!("no support for external types yet"),
-            Type::Custom { name, builtin } => Box::new(custom::CustomCodeType::new(
-                name.clone(),
-                builtin.as_ref().clone(),
-                self.config.custom_types.get(&name).cloned(),
-            )),
+            Type::Optional(inner) => Box::new(compounds::OptionalCodeType::new(*inner)),
+            Type::Sequence(inner) => Box::new(compounds::SequenceCodeType::new(*inner)),
+            Type::Map(key, value) => Box::new(compounds::MapCodeType::new(*key, *value)),
+            Type::External { name, .. } => Box::new(external::ExternalCodeType::new(name)),
+            Type::Custom { name, .. } => Box::new(custom::CustomCodeType::new(name)),
         }
     }
 }
@@ -260,22 +235,22 @@ impl CodeOracle for KotlinCodeOracle {
     }
 
     /// Get the idiomatic Kotlin rendering of a class name (for enums, records, errors, etc).
-    fn class_name(&self, nm: &dyn fmt::Display) -> String {
-        nm.to_string().to_camel_case()
+    fn class_name(&self, nm: &str) -> String {
+        nm.to_string().to_upper_camel_case()
     }
 
     /// Get the idiomatic Kotlin rendering of a function name.
-    fn fn_name(&self, nm: &dyn fmt::Display) -> String {
-        nm.to_string().to_mixed_case()
+    fn fn_name(&self, nm: &str) -> String {
+        nm.to_string().to_lower_camel_case()
     }
 
     /// Get the idiomatic Kotlin rendering of a variable name.
-    fn var_name(&self, nm: &dyn fmt::Display) -> String {
-        nm.to_string().to_mixed_case()
+    fn var_name(&self, nm: &str) -> String {
+        nm.to_string().to_lower_camel_case()
     }
 
     /// Get the idiomatic Kotlin rendering of an individual enum variant.
-    fn enum_variant_name(&self, nm: &dyn fmt::Display) -> String {
+    fn enum_variant_name(&self, nm: &str) -> String {
         nm.to_string().to_shouty_snake_case()
     }
 
@@ -284,7 +259,7 @@ impl CodeOracle for KotlinCodeOracle {
     /// This replaces "Error" at the end of the name with "Exception".  Rust code typically uses
     /// "Error" for any type of error but in the Java world, "Error" means a non-recoverable error
     /// and is distinguished from an "Exception".
-    fn error_name(&self, nm: &dyn fmt::Display) -> String {
+    fn error_name(&self, nm: &str) -> String {
         let name = nm.to_string();
         match name.strip_suffix("Error") {
             None => name,
@@ -317,25 +292,9 @@ impl CodeOracle for KotlinCodeOracle {
 
 pub mod filters {
     use super::*;
-    use std::fmt;
-
-    // This code is a bit unfortunate.  We want to have a `KotlinCodeOracle` instance available for
-    // the filter functions, so that we don't always need to pass as an argument in the template
-    // code.  However, `KotlinCodeOracle` depends on a `Config` instance.  So we use some dirty,
-    // non-threadsafe, code to set it at the start of `generate_kotlin_bindings()`.
-    //
-    // If askama supported using a struct instead of a module for the filters we could avoid this.
-
-    static mut ORACLE: Option<KotlinCodeOracle> = None;
-
-    pub(super) fn set_oracle(oracle: KotlinCodeOracle) {
-        unsafe {
-            ORACLE = Some(oracle);
-        }
-    }
 
     fn oracle() -> &'static KotlinCodeOracle {
-        unsafe { ORACLE.as_ref().unwrap() }
+        &KotlinCodeOracle
     }
 
     pub fn type_name(codetype: &impl CodeType) -> Result<String, askama::Error> {
@@ -354,6 +313,13 @@ pub mod filters {
         Ok(format!("{}.lower", codetype.ffi_converter_name(oracle())))
     }
 
+    pub fn allocation_size_fn(codetype: &impl CodeType) -> Result<String, askama::Error> {
+        Ok(format!(
+            "{}.allocationSize",
+            codetype.ffi_converter_name(oracle())
+        ))
+    }
+
     pub fn write_fn(codetype: &impl CodeType) -> Result<String, askama::Error> {
         Ok(format!("{}.write", codetype.ffi_converter_name(oracle())))
     }
@@ -379,22 +345,22 @@ pub mod filters {
     }
 
     /// Get the idiomatic Kotlin rendering of a class name (for enums, records, errors, etc).
-    pub fn class_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn class_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().class_name(nm))
     }
 
     /// Get the idiomatic Kotlin rendering of a function name.
-    pub fn fn_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn fn_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().fn_name(nm))
     }
 
     /// Get the idiomatic Kotlin rendering of a variable name.
-    pub fn var_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn var_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().var_name(nm))
     }
 
     /// Get the idiomatic Kotlin rendering of an individual enum variant.
-    pub fn enum_variant(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn enum_variant(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().enum_variant_name(nm))
     }
 
@@ -403,7 +369,7 @@ pub mod filters {
     /// This replaces "Error" at the end of the name with "Exception".  Rust code typically uses
     /// "Error" for any type of error but in the Java world, "Error" means a non-recoverable error
     /// and is distinguished from an "Exception".
-    pub fn exception_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn exception_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().error_name(nm))
     }
 }
