@@ -2,11 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-use std::fmt;
-
 use crate::backend::{CodeDeclaration, CodeOracle, CodeType, Literal};
 use crate::interface::{ComponentInterface, Object};
 use askama::Template;
+use std::borrow::Borrow;
 
 // Filters is used by ObjectTemplate.py, which looks for the filters module here.
 use super::filters;
@@ -25,40 +24,14 @@ impl CodeType for ObjectCodeType {
         oracle.class_name(&self.id)
     }
 
-    fn canonical_name(&self, oracle: &dyn CodeOracle) -> String {
-        format!("Object{}", self.type_label(oracle))
+    fn canonical_name(&self, _oracle: &dyn CodeOracle) -> String {
+        format!("Type{}", self.id)
     }
 
     fn literal(&self, _oracle: &dyn CodeOracle, _literal: &Literal) -> String {
         unreachable!();
     }
 
-    fn lower(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-        format!("{}._lower()", oracle.var_name(nm))
-    }
-
-    fn write(
-        &self,
-        oracle: &dyn CodeOracle,
-        nm: &dyn fmt::Display,
-        target: &dyn fmt::Display,
-    ) -> String {
-        format!(
-            "{}._write({}, {})",
-            self.type_label(oracle),
-            oracle.var_name(nm),
-            target
-        )
-    }
-
-    fn lift(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-        format!("{}._lift({})", self.type_label(oracle), nm)
-    }
-
-    fn read(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-        format!("{}._read({})", self.type_label(oracle), nm)
-    }
-
     fn helper_code(&self, oracle: &dyn CodeOracle) -> Option<String> {
         Some(format!(
             "# Helper code for {} class is found in ObjectTemplate.py",
@@ -66,7 +39,7 @@ impl CodeType for ObjectCodeType {
         ))
     }
 
-    fn coerce(&self, _oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
+    fn coerce(&self, _oracle: &dyn CodeOracle, nm: &str) -> String {
         nm.to_string()
     }
 }
