@@ -2,14 +2,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#[allow(unused_imports)]
+use super::filters;
 use crate::backend::{CodeOracle, CodeType, Literal};
 use crate::interface::Radix;
 use askama::Template;
 use paste::paste;
-use std::fmt;
-
-#[allow(unused_imports)]
-use super::filters;
 
 fn render_literal(_oracle: &dyn CodeOracle, literal: &Literal) -> String {
     match literal {
@@ -54,32 +52,16 @@ macro_rules! impl_code_type_for_primitive {
                     render_literal(oracle, &literal)
                 }
 
-                fn lower(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-                    format!("FfiConverter{}._lower({})", $class_name, oracle.var_name(nm))
-                }
-
-                fn write(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display, target: &dyn fmt::Display) -> String {
-                    format!("FfiConverter{}._write({}, {})", $class_name, oracle.var_name(nm), target)
-                }
-
-                fn lift(&self, _oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-                    format!("FfiConverter{}._lift({})", $class_name, nm)
-                }
-
-                fn read(&self, _oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-                    format!("FfiConverter{}._read({})", $class_name, nm)
-                }
-
                 fn helper_code(&self, _oracle: &dyn CodeOracle) -> Option<String> {
                     Some(self.render().unwrap())
                 }
 
-                fn coerce(&self, _oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
+                fn coerce(&self, _oracle: &dyn CodeOracle, nm: &str) -> String {
                     format!($coerce_code, nm)
                 }
             }
         }
-    }
+    };
 }
 
 impl_code_type_for_primitive!(BooleanCodeType, "Bool", "BooleanHelper.py", "bool({})");
