@@ -4,39 +4,40 @@
 class FfiConverterType{{ name }}:
     @staticmethod
     def write(value, buf):
-        {{ "value"|write_var("buf", builtin) }}
+        {{ builtin|ffi_converter_name }}.write(value, buf)
 
     @staticmethod
     def read(buf):
-        return {{ "buf"|read_var(builtin) }}
+        return {{ builtin|ffi_converter_name }}.read(buf)
 
     @staticmethod
     def lift(value):
-        return {{ "value"|lift_var(builtin) }}
+        return {{ builtin|ffi_converter_name }}.lift(value)
 
     @staticmethod
     def lower(value):
-        return {{ "value"|lower_var(builtin) }}
+        return {{ builtin|ffi_converter_name }}.lower(value)
+
 {%- when Some with (config) %}
 {#- Custom type config supplied, use it to convert the builtin type #}
 class FfiConverterType{{ name }}:
     @staticmethod
     def write(value, buf):
         builtin_value = {{ config.from_custom.render("value") }}
-        {{ "builtin_value"|write_var("buf", builtin) }}
+        {{ builtin|write_fn }}(builtin_value, buf)
 
     @staticmethod
     def read(buf):
-        builtin_value = {{ "buf"|read_var(builtin) }}
+        builtin_value = {{ builtin|read_fn }}(buf)
         return {{ config.into_custom.render("builtin_value") }}
 
     @staticmethod
     def lift(value):
-        builtin_value = {{ "value"|lift_var(builtin) }}
+        builtin_value = {{ builtin|lift_fn }}(value)
         return {{ config.into_custom.render("builtin_value") }}
 
     @staticmethod
     def lower(value):
         builtin_value = {{ config.from_custom.render("value") }}
-        return {{ "builtin_value"|lower_var(builtin) }}
+        return {{ builtin|lower_fn }}(builtin_value)
 {%- endmatch %}
