@@ -1,13 +1,13 @@
 {#
 // Template to call into rust. Used in several places.
 // Variable names in `arg_list_decl` should match up with arg lists
-// passed to rust via `_arg_list_ffi_call` (we use  `var_name` in `lower_var`)
+// passed to rust via `_arg_list_ffi_call`
 #}
 
 {%- macro to_ffi_call(func) -%}
-    {%- match func.throws() -%}
+    {%- match func.throws_type() -%}
     {%- when Some with (e) -%}
-rust_call_with_error({{ e|class_name }},
+rust_call_with_error({{ e|ffi_converter_name }},
     {%- else -%}
 rust_call(
     {%- endmatch -%}
@@ -17,10 +17,10 @@ rust_call(
 {%- endmacro -%}
 
 {%- macro to_ffi_call_with_prefix(prefix, func) -%}
-    {%- match func.throws() -%}
+    {%- match func.throws_type() -%}
     {%- when Some with (e) -%}
 rust_call_with_error(
-    {{ e|class_name }},
+    {{ e|ffi_converter_name }},
     {%- else -%}
 rust_call(
     {%- endmatch -%}
@@ -32,7 +32,7 @@ rust_call(
 
 {%- macro _arg_list_ffi_call(func) %}
     {%- for arg in func.arguments() %}
-        {{- arg.name()|lower_var(arg.type_()) }}
+        {{ arg|lower_fn }}({{ arg.name() }})
         {%- if !loop.last %},{% endif %}
     {%- endfor %}
 {%- endmacro -%}
@@ -46,32 +46,47 @@ rust_call(
     {%- for arg in func.arguments() -%}
         {{ arg.name()|var_name }}
         {%- match arg.default_value() %}
-        {%- when Some with(literal) %} = {{ literal|literal_py(arg.type_()) }}
+        {%- when Some with(literal) %} = {{ literal|literal_py(arg.type_().borrow()) }}
         {%- else %}
         {%- endmatch %}
         {%- if !loop.last %},{% endif -%}
     {%- endfor %}
 {%- endmacro %}
 
+{#-
+// Field lists as used in Python declarations of Records.
+// Note the var_name.
+-#}
+{%- macro field_list_decl(item) %}
+    {%- for field in item.fields() -%}
+        {{ field.name()|var_name }}
+        {%- match field.default_value() %}
+            {%- when Some with(literal) %} = {{ literal|literal_py(field) }}
+            {%- else %}
+        {%- endmatch -%}
+        {% if !loop.last %}, {% endif %}
+    {%- endfor %}
+{%- endmacro %}
+
 {#-
 // Arglist as used in the _UniFFILib function declations.
 // Note unfiltered name but ffi_type_name filters.
 -#}
 {%- macro arg_list_ffi_decl(func) %}
     {%- for arg in func.arguments() %}
-    {{ arg.type_()|ffi_type_name }},
+    {{ arg.type_().borrow()|ffi_type_name }},
     {%- endfor %}
     ctypes.POINTER(RustCallStatus),
 {% endmacro -%}
 
 {%- macro coerce_args(func) %}
     {%- for arg in func.arguments() %}
-    {{ arg.name() }} = {{ arg.name()|coerce_py(arg.type_()) -}}
+    {{ arg.name() }} = {{ arg.name()|coerce_py(arg.type_().borrow()) -}}
     {% endfor -%}
 {%- endmacro -%}
 
 {%- macro coerce_args_extra_indent(func) %}
         {%- for arg in func.arguments() %}
-        {{ arg.name() }} = {{ arg.name()|coerce_py(arg.type_()) }}
+        {{ arg.name() }} = {{ arg.name()|coerce_py(arg.type_().borrow()) }}
         {%- endfor %}
 {%- endmacro -%}
