@@ -4,8 +4,9 @@
 
 use anyhow::Result;
 use askama::Template;
-use heck::{CamelCase, ShoutySnakeCase, SnakeCase};
+use heck::{ToShoutySnakeCase, ToSnakeCase, ToUpperCamelCase};
 use serde::{Deserialize, Serialize};
+use std::borrow::Borrow;
 
 use crate::interface::*;
 use crate::MergeWith;
@@ -78,7 +79,6 @@ impl<'a> RubyWrapper<'a> {
 
 mod filters {
     use super::*;
-    use std::fmt;
 
     pub fn type_ffi(type_: &FFIType) -> Result<String, askama::Error> {
         Ok(match type_ {
@@ -132,26 +132,26 @@ mod filters {
         })
     }
 
-    pub fn class_name_rb(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
-        Ok(nm.to_string().to_camel_case())
+    pub fn class_name_rb(nm: &str) -> Result<String, askama::Error> {
+        Ok(nm.to_string().to_upper_camel_case())
     }
 
-    pub fn fn_name_rb(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn fn_name_rb(nm: &str) -> Result<String, askama::Error> {
         Ok(nm.to_string().to_snake_case())
     }
 
-    pub fn var_name_rb(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn var_name_rb(nm: &str) -> Result<String, askama::Error> {
         let nm = nm.to_string();
         let prefix = if is_reserved_word(&nm) { "_" } else { "" };
 
         Ok(format!("{}{}", prefix, nm.to_snake_case()))
     }
 
-    pub fn enum_name_rb(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn enum_name_rb(nm: &str) -> Result<String, askama::Error> {
         Ok(nm.to_string().to_shouty_snake_case())
     }
 
-    pub fn coerce_rb(nm: &dyn fmt::Display, type_: &Type) -> Result<String, askama::Error> {
+    pub fn coerce_rb(nm: &str, type_: &Type) -> Result<String, askama::Error> {
         Ok(match type_ {
             Type::Int8
             | Type::UInt8
@@ -170,16 +170,16 @@ mod filters {
             Type::CallbackInterface(_) => panic!("No support for coercing callback interfaces yet"),
             Type::Optional(t) => format!("({} ? {} : nil)", nm, coerce_rb(nm, t)?),
             Type::Sequence(t) => {
-                let coerce_code = coerce_rb(&"v", t)?;
+                let coerce_code = coerce_rb("v", t)?;
                 if coerce_code == "v" {
                     nm.to_string()
                 } else {
                     format!("{}.map {{ |v| {} }}", nm, coerce_code)
                 }
             }
-            Type::Map(t) => {
-                let k_coerce_code = coerce_rb(&"k", &Type::String)?;
-                let v_coerce_code = coerce_rb(&"v", t)?;
+            Type::Map(_k, t) => {
+                let k_coerce_code = coerce_rb("k", &Type::String)?;
+                let v_coerce_code = coerce_rb("v", t)?;
 
                 if k_coerce_code == "k" && v_coerce_code == "v" {
                     nm.to_string()
@@ -195,7 +195,7 @@ mod filters {
         })
     }
 
-    pub fn lower_rb(nm: &dyn fmt::Display, type_: &Type) -> Result<String, askama::Error> {
+    pub fn lower_rb(nm: &str, type_: &Type) -> Result<String, askama::Error> {
         Ok(match type_ {
             Type::Int8
             | Type::UInt8
@@ -218,7 +218,7 @@ mod filters {
             | Type::Record(_)
             | Type::Optional(_)
             | Type::Sequence(_)
-            | Type::Map(_) => format!(
+            | Type::Map(_, _) => format!(
                 "RustBuffer.alloc_from_{}({})",
                 class_name_rb(&type_.canonical_name())?,
                 nm
@@ -228,7 +228,7 @@ mod filters {
         })
     }
 
-    pub fn lift_rb(nm: &dyn fmt::Display, type_: &Type) -> Result<String, askama::Error> {
+    pub fn lift_rb(nm: &str, type_: &Type) -> Result<String, askama::Error> {
         Ok(match type_ {
             Type::Int8
             | Type::UInt8
@@ -250,7 +250,7 @@ mod filters {
             | Type::Record(_)
             | Type::Optional(_)
             | Type::Sequence(_)
-            | Type::Map(_) => format!(
+            | Type::Map(_, _) => format!(
                 "{}.consumeInto{}",
                 nm,
                 class_name_rb(&type_.canonical_name())?
