@@ -3,10 +3,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 use crate::backend::{CodeOracle, CodeType, Literal, TypeIdentifier};
-use crate::interface::types::Type;
 use askama::Template;
 use paste::paste;
-use std::fmt;
 
 // Used in template files.
 use super::filters;
@@ -42,10 +40,6 @@ macro_rules! impl_code_type_for_compound {
                 fn outer(&self) -> &TypeIdentifier {
                     &self.outer
                 }
-
-                fn ffi_converter_name(&self, oracle: &dyn CodeOracle) -> String {
-                    format!("FfiConverter{}", self.canonical_name(oracle))
-                }
             }
 
             impl CodeType for $T  {
@@ -58,23 +52,7 @@ macro_rules! impl_code_type_for_compound {
                 }
 
                 fn literal(&self, oracle: &dyn CodeOracle, literal: &Literal) -> String {
-                    render_literal(oracle, &literal, self.inner())
-                }
-
-                fn lower(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-                    format!("{}.lower({})", self.ffi_converter_name(oracle), oracle.var_name(nm))
-                }
-
-                fn write(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display, target: &dyn fmt::Display) -> String {
-                    format!("{}.write({}, into: {})", self.ffi_converter_name(oracle), oracle.var_name(nm), target)
-                }
-
-                fn lift(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-                    format!("{}.lift({})", self.ffi_converter_name(oracle), nm)
-                }
-
-                fn read(&self, oracle: &dyn CodeOracle, nm: &dyn fmt::Display) -> String {
-                    format!("{}.read(from: {})", self.ffi_converter_name(oracle), nm)
+                    render_literal(oracle, literal, self.inner())
                 }
 
                 fn helper_code(&self, _oracle: &dyn CodeOracle) -> Option<String> {
@@ -99,9 +77,54 @@ impl_code_type_for_compound!(
     "SequenceTemplate.swift"
 );
 
-impl_code_type_for_compound!(
-    MapCodeType,
-    "[String: {}]",
-    "Dictionary{}",
-    "MapTemplate.swift"
-);
+#[derive(Template)]
+#[template(syntax = "swift", escape = "none", path = "MapTemplate.swift")]
+pub struct MapCodeType {
+    key: TypeIdentifier,
+    value: TypeIdentifier,
+    outer: TypeIdentifier,
+}
+
+impl MapCodeType {
+    pub fn new(key: TypeIdentifier, value: TypeIdentifier, outer: TypeIdentifier) -> Self {
+        Self { key, value, outer }
+    }
+
+    fn key(&self) -> &TypeIdentifier {
+        &self.key
+    }
+
+    fn value(&self) -> &TypeIdentifier {
+        &self.value
+    }
+
+    fn outer(&self) -> &TypeIdentifier {
+        &self.outer
+    }
+}
+
+impl CodeType for MapCodeType {
+    fn type_label(&self, oracle: &dyn CodeOracle) -> String {
+        format!(
+            "[{}: {}]",
+            oracle.find(self.key()).type_label(oracle),
+            oracle.find(self.value()).type_label(oracle)
+        )
+    }
+
+    fn canonical_name(&self, oracle: &dyn CodeOracle) -> String {
+        format!(
+            "Dictionary{}{}",
+            oracle.find(self.key()).type_label(oracle),
+            oracle.find(self.value()).type_label(oracle)
+        )
+    }
+
+    fn literal(&self, oracle: &dyn CodeOracle, literal: &Literal) -> String {
+        render_literal(oracle, literal, self.value())
+    }
+
+    fn helper_code(&self, _oracle: &dyn CodeOracle) -> Option<String> {
+        Some(self.render().unwrap())
+    }
+}
