@@ -2,12 +2,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+use std::borrow::Borrow;
 use std::collections::{HashMap, HashSet};
-use std::fmt;
 
 use anyhow::{anyhow, Result};
 use askama::Template;
-use heck::{CamelCase, MixedCase};
+use heck::{ToLowerCamelCase, ToUpperCamelCase};
 use serde::{Deserialize, Serialize};
 
 use super::Bindings;
@@ -138,13 +138,10 @@ impl MergeWith for Config {
 /// Generate UniFFI component bindings for Swift, as strings in memory.
 ///
 pub fn generate_bindings(config: &Config, ci: &ComponentInterface) -> Result<Bindings> {
-    let oracle = SwiftCodeOracle::new(config.clone());
-    filters::set_oracle(oracle.clone());
-
     let header = BridgingHeader::new(config, ci)
         .render()
         .map_err(|_| anyhow!("failed to render Swift bridging header"))?;
-    let library = SwiftWrapper::new(oracle, config.clone(), ci)
+    let library = SwiftWrapper::new(SwiftCodeOracle, config.clone(), ci)
         .render()
         .map_err(|_| anyhow!("failed to render Swift library"))?;
     let modulemap = if config.generate_module_map() {
@@ -218,7 +215,6 @@ impl<'a> SwiftWrapper<'a> {
     pub fn members(&self) -> Vec<Box<dyn CodeDeclaration + 'a>> {
         let ci = self.ci;
         vec![
-            Box::new(object::SwiftObjectRuntime::new(ci)) as Box<dyn CodeDeclaration>,
             Box::new(callback_interface::SwiftCallbackInterfaceRuntime::new(ci))
                 as Box<dyn CodeDeclaration>,
         ]
@@ -310,15 +306,9 @@ impl<'a> SwiftWrapper<'a> {
 }
 
 #[derive(Clone)]
-pub struct SwiftCodeOracle {
-    config: Config,
-}
+pub struct SwiftCodeOracle;
 
 impl SwiftCodeOracle {
-    fn new(config: Config) -> Self {
-        Self { config }
-    }
-
     fn create_code_type(&self, type_: TypeIdentifier) -> Box<dyn CodeType> {
         // I really want access to the ComponentInterface here so I can look up the interface::{Enum, Record, Error, Object, etc}
         // However, there's some violence and gore I need to do to (temporarily) make the oracle usable from filters.
@@ -359,17 +349,14 @@ impl SwiftCodeOracle {
                 let inner = *inner.to_owned();
                 Box::new(compounds::SequenceCodeType::new(inner, outer))
             }
-            Type::Map(ref inner) => {
+            Type::Map(ref key, ref value) => {
                 let outer = type_.clone();
-                let inner = *inner.to_owned();
-                Box::new(compounds::MapCodeType::new(inner, outer))
+                let key = *key.to_owned();
+                let value = *value.to_owned();
+                Box::new(compounds::MapCodeType::new(key, value, outer))
             }
             Type::External { .. } => panic!("no support for external types yet"),
-            Type::Custom { name, builtin } => Box::new(custom::CustomCodeType::new(
-                name.clone(),
-                builtin.as_ref().clone(),
-                self.config.custom_types.get(&name).cloned(),
-            )),
+            Type::Custom { name, .. } => Box::new(custom::CustomCodeType::new(name)),
         }
     }
 }
@@ -380,27 +367,27 @@ impl CodeOracle for SwiftCodeOracle {
     }
 
     /// Get the idiomatic Swift rendering of a class name (for enums, records, errors, etc).
-    fn class_name(&self, nm: &dyn fmt::Display) -> String {
-        nm.to_string().to_camel_case()
+    fn class_name(&self, nm: &str) -> String {
+        nm.to_string().to_upper_camel_case()
     }
 
     /// Get the idiomatic Swift rendering of a function name.
-    fn fn_name(&self, nm: &dyn fmt::Display) -> String {
-        nm.to_string().to_mixed_case()
+    fn fn_name(&self, nm: &str) -> String {
+        nm.to_string().to_lower_camel_case()
     }
 
     /// Get the idiomatic Swift rendering of a variable name.
-    fn var_name(&self, nm: &dyn fmt::Display) -> String {
-        nm.to_string().to_mixed_case()
+    fn var_name(&self, nm: &str) -> String {
+        nm.to_string().to_lower_camel_case()
     }
 
     /// Get the idiomatic Swift rendering of an individual enum variant.
-    fn enum_variant_name(&self, nm: &dyn fmt::Display) -> String {
-        nm.to_string().to_mixed_case()
+    fn enum_variant_name(&self, nm: &str) -> String {
+        nm.to_string().to_lower_camel_case()
     }
 
     /// Get the idiomatic Swift rendering of an exception name.
-    fn error_name(&self, nm: &dyn fmt::Display) -> String {
+    fn error_name(&self, nm: &str) -> String {
         self.class_name(nm)
     }
 
@@ -426,25 +413,9 @@ impl CodeOracle for SwiftCodeOracle {
 
 pub mod filters {
     use super::*;
-    use std::fmt;
-
-    // This code is a bit unfortunate.  We want to have a `SwiftCodeOracle` instance available for
-    // the filter functions, so that we don't always need to pass as an argument in the template
-    // code.  However, `SwiftCodeOracle` depends on a `Config` instance.  So we use some dirty,
-    // non-threadsafe, code to set it at the start of `generate_bindings()`.
-    //
-    // If askama supported using a struct instead of a module for the filters we could avoid this.
-
-    static mut ORACLE: Option<SwiftCodeOracle> = None;
-
-    pub(super) fn set_oracle(oracle: SwiftCodeOracle) {
-        unsafe {
-            ORACLE = Some(oracle);
-        }
-    }
 
     fn oracle() -> &'static SwiftCodeOracle {
-        unsafe { ORACLE.as_ref().unwrap() }
+        &SwiftCodeOracle
     }
 
     pub fn type_name(codetype: &impl CodeType) -> Result<String, askama::Error> {
@@ -457,29 +428,24 @@ pub mod filters {
         Ok(codetype.canonical_name(oracle))
     }
 
-    pub fn lower_var(
-        nm: &dyn fmt::Display,
-        codetype: &impl CodeType,
-    ) -> Result<String, askama::Error> {
-        let oracle = oracle();
-        Ok(codetype.lower(oracle, nm))
+    pub fn ffi_converter_name(codetype: &impl CodeType) -> Result<String, askama::Error> {
+        Ok(codetype.ffi_converter_name(oracle()))
     }
 
-    pub fn write_var(
-        nm: &dyn fmt::Display,
-        target: &dyn fmt::Display,
-        codetype: &impl CodeType,
-    ) -> Result<String, askama::Error> {
-        let oracle = oracle();
-        Ok(codetype.write(oracle, nm, target))
+    pub fn lower_fn(codetype: &impl CodeType) -> Result<String, askama::Error> {
+        Ok(format!("{}.lower", codetype.ffi_converter_name(oracle())))
     }
 
-    pub fn lift_var(
-        nm: &dyn fmt::Display,
-        codetype: &impl CodeType,
-    ) -> Result<String, askama::Error> {
-        let oracle = oracle();
-        Ok(codetype.lift(oracle, nm))
+    pub fn write_fn(codetype: &impl CodeType) -> Result<String, askama::Error> {
+        Ok(format!("{}.write", codetype.ffi_converter_name(oracle())))
+    }
+
+    pub fn lift_fn(codetype: &impl CodeType) -> Result<String, askama::Error> {
+        Ok(format!("{}.lift", codetype.ffi_converter_name(oracle())))
+    }
+
+    pub fn read_fn(codetype: &impl CodeType) -> Result<String, askama::Error> {
+        Ok(format!("{}.read", codetype.ffi_converter_name(oracle())))
     }
 
     pub fn literal_swift(
@@ -490,14 +456,6 @@ pub mod filters {
         Ok(codetype.literal(oracle, literal))
     }
 
-    pub fn read_var(
-        nm: &dyn fmt::Display,
-        codetype: &impl CodeType,
-    ) -> Result<String, askama::Error> {
-        let oracle = oracle();
-        Ok(codetype.read(oracle, nm))
-    }
-
     /// Get the Swift syntax for representing a given low-level `FFIType`.
     pub fn ffi_type_name(type_: &FFIType) -> Result<String, askama::Error> {
         Ok(oracle().ffi_type_label(type_))
@@ -525,22 +483,22 @@ pub mod filters {
     }
 
     /// Get the idiomatic Swift rendering of a class name (for enums, records, errors, etc).
-    pub fn class_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn class_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().class_name(nm))
     }
 
     /// Get the idiomatic Swift rendering of a function name.
-    pub fn fn_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn fn_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().fn_name(nm))
     }
 
     /// Get the idiomatic Swift rendering of a variable name.
-    pub fn var_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn var_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().var_name(nm))
     }
 
     /// Get the idiomatic Swift rendering of an individual enum variant.
-    pub fn enum_variant_swift(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn enum_variant_swift(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().enum_variant_name(nm))
     }
 
@@ -549,7 +507,7 @@ pub mod filters {
     /// This replaces "Error" at the end of the name with "Exception".  Rust code typically uses
     /// "Error" for any type of error but in the Java world, "Error" means a non-recoverable error
     /// and is distinguished from an "Exception".
-    pub fn exception_name(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+    pub fn exception_name(nm: &str) -> Result<String, askama::Error> {
         Ok(oracle().error_name(nm))
     }
 }
