@@ -2,7 +2,7 @@
 {%- let cbi = self.inner() %}
 {%- let type_name = cbi|type_name %}
 {%- let canonical_type_name = cbi|canonical_name %}
-{%- let ffi_converter = format!("ffiConverter{}", canonical_type_name) %}
+{%- let ffi_converter = cbi|ffi_converter_name %}
 {%- let foreign_callback = format!("foreignCallback{}", canonical_type_name) %}
 
 // Declaration and FfiConverters for {{ type_name }} Callback Interface
@@ -34,7 +34,7 @@ fileprivate let {{ foreign_callback }} : ForeignCallback =
             {% if meth.throws().is_some() %}try {% endif -%}
             swiftCallbackInterface.{{ meth.name()|fn_name }}(
                     {% for arg in meth.arguments() -%}
-                    {{ arg.name() }}: try {{ "reader"|read_var(arg) }}
+                    {{ arg.name() }}: try {{ arg|read_fn }}(from: reader)
                     {%- if !loop.last %}, {% endif %}
                     {% endfor -%}
                 )
@@ -48,7 +48,7 @@ fileprivate let {{ foreign_callback }} : ForeignCallback =
                 {%- match meth.return_type() -%}
                 {%- when Some with (return_type) -%}
                 let writer = Writer()
-                {{ "result"|write_var("writer", return_type) }}
+                {{ return_type|write_fn }}(result, into: writer)
                 return RustBuffer(bytes: writer.bytes)
                 {%- else -%}
                 return RustBuffer()
@@ -85,10 +85,55 @@ fileprivate let {{ foreign_callback }} : ForeignCallback =
         }
     }
 
-// The ffiConverter which transforms the Callbacks in to Handles to pass to Rust.
-private let {{ ffi_converter }}: FfiConverterCallbackInterface<{{ type_name }}> = {
-    try! rustCall { (err: UnsafeMutablePointer<RustCallStatus>) in
-            {{ cbi.ffi_init_callback().name() }}({{ foreign_callback }}, err)
+// FFIConverter protocol for callback interfaces
+fileprivate struct {{ ffi_converter }} {
+    // Initialize our callback method with the scaffolding code
+    private static var callbackInitialized = false
+    private static func initCallback() {
+        try! rustCall { (err: UnsafeMutablePointer<RustCallStatus>) in
+                {{ cbi.ffi_init_callback().name() }}({{ foreign_callback }}, err)
+        }
     }
-    return FfiConverterCallbackInterface<{{ type_name }}>()
-}()
+    private static func ensureCallbackinitialized() {
+        if !callbackInitialized {
+            initCallback()
+            callbackInitialized = true
+        }
+    }
+
+    static func drop(handle: Handle) {
+        handleMap.remove(handle: handle)
+    }
+
+    private static var handleMap = ConcurrentHandleMap<{{ type_name }}>()
+}
+
+extension {{ ffi_converter }} : FfiConverter {
+    typealias SwiftType = {{ type_name }}
+    // We can use Handle as the FFIType because it's a typealias to UInt64
+    typealias FfiType = Handle
+
+    static func lift(_ handle: Handle) throws -> SwiftType {
+        ensureCallbackinitialized();
+        guard let callback = handleMap.get(handle: handle) else {
+            throw UniffiInternalError.unexpectedStaleHandle
+        }
+        return callback
+    }
+
+    static func read(from buf: Reader) throws -> SwiftType {
+        ensureCallbackinitialized();
+        let handle: Handle = try buf.readInt()
+        return try lift(handle)
+    }
+
+    static func lower(_ v: SwiftType) -> Handle {
+        ensureCallbackinitialized();
+        return handleMap.insert(obj: v)
+    }
+
+    static func write(_ v: SwiftType, into buf: Writer) {
+        ensureCallbackinitialized();
+        buf.writeInt(lower(v))
+    }
+}
