@@ -1,22 +1,24 @@
-extension TimeInterval: ViaFfiUsingByteBuffer, ViaFfi {
-    fileprivate static func read(from buf: Reader) throws -> Self {
+fileprivate struct FfiConverterDuration: FfiConverterRustBuffer {
+    typealias SwiftType = TimeInterval
+
+    static func read(from buf: Reader) throws -> TimeInterval {
         let seconds: UInt64 = try buf.readInt()
         let nanoseconds: UInt32 = try buf.readInt()
         return Double(seconds) + (Double(nanoseconds) / 1.0e9)
     }
 
-    fileprivate func write(into buf: Writer) {
-        if self.rounded(.down) > Double(Int64.max) {
+    static func write(_ value: TimeInterval, into buf: Writer) {
+        if value.rounded(.down) > Double(Int64.max) {
             fatalError("Duration overflow, exceeds max bounds supported by Uniffi")
         }
 
-        if self < 0 {
+        if value < 0 {
             fatalError("Invalid duration, must be non-negative")
         }
 
-        let seconds = UInt64(self)
-        let nanoseconds = UInt32((self - Double(seconds)) * 1.0e9)
+        let seconds = UInt64(value)
+        let nanoseconds = UInt32((value - Double(seconds)) * 1.0e9)
         buf.writeInt(seconds)
         buf.writeInt(nanoseconds)
     }
-}
\ No newline at end of file
+}
