@@ -4,8 +4,9 @@
 {%- when Some with (return_type) %}
 
 public func {{ func.name()|fn_name }}({%- call swift::arg_list_decl(func) -%}) {% call swift::throws(func) %} -> {{ return_type|type_name }} {
-    let _retval = {% call swift::to_ffi_call(func) %}
-    return {% call swift::try(func) %} {{ "_retval"|lift_var(return_type) }}
+    return {% call swift::try(func) %} {{ return_type|lift_fn }}(
+        {% call swift::to_ffi_call(func) %}
+    )
 }
 
 {% when None -%}
