@@ -132,6 +132,14 @@ impl Object {
         self.methods.iter().collect()
     }
 
+    pub fn get_method(&self, name: &str) -> Method {
+        let matches: Vec<_> = self.methods.iter().filter(|m| m.name() == name).collect();
+        match matches.len() {
+            1 => matches[0].clone(),
+            n => panic!("{} methods named {}", n, name),
+        }
+    }
+
     pub fn ffi_object_free(&self) -> &FFIFunction {
         &self.ffi_func_free
     }
@@ -267,6 +275,10 @@ impl Constructor {
             .map(|name| Type::Error(name.to_owned()))
     }
 
+    pub fn is_primary_constructor(&self) -> bool {
+        self.name == "new"
+    }
+
     fn derive_ffi_func(&mut self, ci_prefix: &str, obj_prefix: &str) {
         self.ffi_func.name.push_str(ci_prefix);
         self.ffi_func.name.push('_');
@@ -276,15 +288,11 @@ impl Constructor {
         self.ffi_func.arguments = self.arguments.iter().map(Into::into).collect();
         self.ffi_func.return_type = Some(FFIType::RustArcPtr);
     }
-
-    fn is_primary_constructor(&self) -> bool {
-        self.name == "new"
-    }
 }
 
 impl IterTypes for Constructor {
     fn iter_types(&self) -> TypeIterator<'_> {
-        Box::new(self.arguments.iter().map(IterTypes::iter_types).flatten())
+        Box::new(self.arguments.iter().flat_map(IterTypes::iter_types))
     }
 }
 
@@ -407,8 +415,7 @@ impl IterTypes for Method {
         Box::new(
             self.arguments
                 .iter()
-                .map(IterTypes::iter_types)
-                .flatten()
+                .flat_map(IterTypes::iter_types)
                 .chain(self.return_type.iter_types()),
         )
     }
