@@ -119,12 +119,22 @@ impl TypeResolver for weedle::types::SequenceType<'_> {
     }
 }
 
+impl TypeResolver for weedle::types::RecordKeyType<'_> {
+    fn resolve_type_expression(&self, types: &mut TypeUniverse) -> Result<Type> {
+        use weedle::types::RecordKeyType::*;
+        match self {
+            Byte(_) | USV(_) => bail!("WebIDL Byte or USV string type not implemented ({:?}); consider using DOMString or string", self),
+            DOM(_) => types.add_known_type(Type::String),
+            NonAny(t) => t.resolve_type_expression(types)
+        }
+    }
+}
+
 impl TypeResolver for weedle::types::RecordType<'_> {
     fn resolve_type_expression(&self, types: &mut TypeUniverse) -> Result<Type> {
-        let t = (&self.generics.body.2).resolve_type_expression(types)?;
-        // Maps always have string keys, make sure the `String` type is known.
-        types.add_known_type(Type::String)?;
-        types.add_known_type(Type::Map(Box::new(t)))
+        let key_type = (&self.generics.body.0).resolve_type_expression(types)?;
+        let value_type = (&self.generics.body.2).resolve_type_expression(types)?;
+        types.add_known_type(Type::Map(Box::new(key_type), Box::new(value_type)))
     }
 }
 
@@ -251,11 +261,11 @@ mod test {
         assert_eq!(types.iter_known_types().count(), 0);
         let (_, expr) = weedle::types::Type::parse("record<DOMString, float>").unwrap();
         let t = types.resolve_type_expression(expr).unwrap();
-        assert_eq!(t.canonical_name(), "Mapf32");
+        assert_eq!(t.canonical_name(), "MapStringF32");
         assert_eq!(types.iter_known_types().count(), 3);
         assert!(types
             .iter_known_types()
-            .any(|t| t.canonical_name() == "Mapf32"));
+            .any(|t| t.canonical_name() == "MapStringF32"));
         assert!(types
             .iter_known_types()
             .any(|t| t.canonical_name() == "string"));
@@ -264,6 +274,25 @@ mod test {
             .any(|t| t.canonical_name() == "f32"));
     }
 
+    #[test]
+    fn test_resolving_map_type_adds_key_type_and_inner_type() {
+        let mut types = TypeUniverse::default();
+        assert_eq!(types.iter_known_types().count(), 0);
+        let (_, expr) = weedle::types::Type::parse("record<u64, float>").unwrap();
+        let t = types.resolve_type_expression(expr).unwrap();
+        assert_eq!(t.canonical_name(), "MapU64F32");
+        assert_eq!(types.iter_known_types().count(), 3);
+        assert!(types
+            .iter_known_types()
+            .any(|t| t.canonical_name() == "MapU64F32"));
+        assert!(types
+            .iter_known_types()
+            .any(|t| t.canonical_name() == "u64"));
+        assert!(types
+            .iter_known_types()
+            .any(|t| t.canonical_name() == "f32"));
+    }
+
     #[test]
     fn test_error_on_unknown_type() -> Result<()> {
         let mut types = TypeUniverse::default();
