@@ -95,15 +95,18 @@
 const BINDGEN_VERSION: &str = env!("CARGO_PKG_VERSION");
 
 use anyhow::{anyhow, bail, Context, Result};
+use clap::{Parser, Subcommand};
 use serde::{Deserialize, Serialize};
 use std::convert::TryInto;
 use std::io::prelude::*;
 use std::{
     collections::HashMap,
     env,
+    ffi::OsString,
     fs::File,
     path::{Path, PathBuf},
     process::Command,
+    str::FromStr,
 };
 
 pub mod backend;
@@ -112,9 +115,157 @@ pub mod interface;
 pub mod scaffolding;
 
 use bindings::TargetLanguage;
-use interface::ComponentInterface;
+pub use interface::ComponentInterface;
 use scaffolding::RustScaffolding;
 
+/// A trait representing a Binding Generator Configuration
+///
+/// External crates that implement binding generators need to implement this trait and set it as
+/// the `BindingGenerator.config` associated type.  `generate_external_bindings()` then uses it to
+/// generate the config that's passed to `BindingGenerator.write_bindings()`
+pub trait BindingGeneratorConfig: for<'de> Deserialize<'de> {
+    /// Get the entry for this config from the `bindings` table.
+    fn get_entry_from_bindings_table(bindings: &toml::Value) -> Option<toml::Value>;
+
+    /// Get default config values from the `ComponentInterface`
+    ///
+    /// These will replace missing entries in the bindings-specific config
+    fn get_config_defaults(ci: &ComponentInterface) -> Vec<(String, toml::Value)>;
+}
+
+fn load_bindings_config<BC: BindingGeneratorConfig>(
+    ci: &ComponentInterface,
+    udl_file: &Path,
+    config_file_override: Option<&Path>,
+) -> Result<BC> {
+    // Load the config from the TOML value, falling back to an empty map if it doesn't exist
+    let mut config_map: toml::value::Table =
+        match load_bindings_config_toml::<BC>(udl_file, config_file_override)? {
+            Some(value) => value
+                .try_into()
+                .context("Bindings config must be a TOML table")?,
+            None => toml::map::Map::new(),
+        };
+
+    // Update it with the defaults from the component interface
+    for (key, value) in BC::get_config_defaults(ci).into_iter() {
+        config_map.entry(key).or_insert(value);
+    }
+
+    // Leverage serde to convert toml::Value into the config type
+    toml::Value::from(config_map)
+        .try_into()
+        .context("Generating bindings config from toml::Value")
+}
+
+/// Binding generator config with no members
+#[derive(Clone, Debug, Hash, PartialEq, PartialOrd, Ord, Eq)]
+pub struct EmptyBindingGeneratorConfig;
+
+impl BindingGeneratorConfig for EmptyBindingGeneratorConfig {
+    fn get_entry_from_bindings_table(_bindings: &toml::Value) -> Option<toml::Value> {
+        None
+    }
+
+    fn get_config_defaults(_ci: &ComponentInterface) -> Vec<(String, toml::Value)> {
+        Vec::new()
+    }
+}
+
+// EmptyBindingGeneratorConfig is a unit struct, so the `derive(Deserialize)` implementation
+// expects a null value rather than the empty map that we pass it.  So we need to implement
+// `Deserialize` ourselves.
+impl<'de> Deserialize<'de> for EmptyBindingGeneratorConfig {
+    fn deserialize<D>(_deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        Ok(EmptyBindingGeneratorConfig)
+    }
+}
+
+// Load the binding-specific config
+//
+// This function calulates the location of the config TOML file, parses it, and returns the result
+// as a toml::Value
+//
+// If there is an error parsing the file then Err will be returned. If the file is missing or the
+// entry for the bindings is missing, then Ok(None) will be returned.
+fn load_bindings_config_toml<BC: BindingGeneratorConfig>(
+    udl_file: &Path,
+    config_file_override: Option<&Path>,
+) -> Result<Option<toml::Value>> {
+    let config_path = match config_file_override {
+        Some(cfg) => cfg.to_owned(),
+        None => guess_crate_root(udl_file)?.join("uniffi.toml"),
+    };
+
+    if !config_path.exists() {
+        return Ok(None);
+    }
+
+    let contents = slurp_file(&config_path)
+        .with_context(|| format!("Failed to read config file from {:?}", config_path))?;
+    let full_config = toml::Value::from_str(&contents)
+        .with_context(|| format!("Failed to parse config file {:?}", config_path))?;
+
+    Ok(full_config
+        .get("bindings")
+        .and_then(BC::get_entry_from_bindings_table))
+}
+
+/// A trait representing a UniFFI Binding Generator
+///
+/// External crates that implement binding generators, should implement this type
+/// and call the [`generate_external_bindings`] using a type that implements this trait.
+pub trait BindingGenerator: Sized {
+    /// Associated type representing a the bindings-specifig configuration parsed from the
+    /// uniffi.toml
+    type Config: BindingGeneratorConfig;
+
+    /// Writes the bindings to the output directory
+    ///
+    /// # Arguments
+    /// - `ci`: A [`ComponentInterface`] representing the interface
+    /// - `config`: A instance of the BindingGeneratorConfig associated with this type
+    /// - `out_dir`: The path to where the binding generator should write the output bindings
+    fn write_bindings(
+        &self,
+        ci: ComponentInterface,
+        config: Self::Config,
+        out_dir: &Path,
+    ) -> anyhow::Result<()>;
+}
+
+/// Generate bindings for an external binding generator
+/// Ideally, this should replace the [`generate_bindings`] function below
+///
+/// Implements an entry point for external binding generators.
+/// The function does the following:
+/// - It parses the `udl` in a [`ComponentInterface`]
+/// - Parses the `uniffi.toml` and loads it into the type that implements [`BindingGeneratorConfig`]
+/// - Creates an instance of [`BindingGenerator`], based on type argument `B`, and run [`BindingGenerator::write_bindings`] on it
+///
+/// # Arguments
+/// - `binding_generator`: Type that implements BindingGenerator
+/// - `udl_file`: The path to the UDL file
+/// - `config_file_override`: The path to the configuration toml file, most likely called `uniffi.toml`. If [`None`], the function will try to guess based on the crate's root.
+/// - `out_dir_override`: The path to write the bindings to. If [`None`], it will be the path to the parent directory of the `udl_file`
+pub fn generate_external_bindings(
+    binding_generator: impl BindingGenerator,
+    udl_file: impl AsRef<Path>,
+    config_file_override: Option<impl AsRef<Path>>,
+    out_dir_override: Option<impl AsRef<Path>>,
+) -> Result<()> {
+    let out_dir_override = out_dir_override.as_ref().map(|p| p.as_ref());
+    let config_file_override = config_file_override.as_ref().map(|p| p.as_ref());
+    let out_dir = get_out_dir(udl_file.as_ref(), out_dir_override)?;
+    let component = parse_udl(udl_file.as_ref()).context("Error parsing UDL")?;
+    let bindings_config =
+        load_bindings_config(&component, udl_file.as_ref(), config_file_override)?;
+    binding_generator.write_bindings(component, bindings_config, out_dir.as_path())
+}
+
 // Generate the infrastructural Rust code for implementing the UDL interface,
 // such as the `extern "C"` function definitions and record data types.
 pub fn generate_component_scaffolding<P: AsRef<Path>>(
@@ -344,112 +495,133 @@ impl<V: Clone> MergeWith for HashMap<String, V> {
     }
 }
 
+// structs to help our cmdline parsing.
+#[derive(Parser)]
+#[clap(name = "uniffi-bindgen")]
+#[clap(version = clap::crate_version!())]
+#[clap(about = "Scaffolding and bindings generator for Rust")]
+#[clap(propagate_version = true)]
+struct Cli {
+    #[clap(subcommand)]
+    command: Commands,
+}
+
+#[derive(Subcommand)]
+enum Commands {
+    #[clap(name = "generate", about = "Generate foreign language bindings")]
+    Generate {
+        #[clap(long, short, possible_values = &["kotlin", "python", "swift", "ruby"])]
+        #[clap(help = "Foreign language(s) for which to build bindings.")]
+        language: Vec<String>,
+
+        #[clap(
+            long,
+            short,
+            help = "Directory in which to write generated files. Default is same folder as .udl file."
+        )]
+        out_dir: Option<OsString>,
+
+        #[clap(long, short, help = "Do not try to format the generated bindings.")]
+        no_format: bool,
+
+        #[clap(
+            long,
+            short,
+            help = "Path to the optional uniffi config file. If not provided, uniffi-bindgen will try to guess it from the UDL's file location."
+        )]
+        config: Option<OsString>,
+
+        #[clap(help = "Path to the UDL file.")]
+        udl_file: OsString,
+    },
+
+    #[clap(name = "scaffolding", about = "Generate Rust scaffolding code")]
+    Scaffolding {
+        #[clap(
+            long,
+            short,
+            help = "Directory in which to write generated files. Default is same folder as .udl file."
+        )]
+        out_dir: Option<OsString>,
+
+        #[clap(
+            long,
+            short,
+            help = "Path to the optional uniffi config file. If not provided, uniffi-bindgen will try to guess it from the UDL's file location."
+        )]
+        config: Option<OsString>,
+
+        #[clap(long, short, help = "Do not try to format the generated bindings.")]
+        no_format: bool,
+
+        #[clap(help = "Path to the UDL file.")]
+        udl_file: OsString,
+    },
+
+    #[clap(
+        name = "test",
+        about = "Run test scripts against foreign language bindings."
+    )]
+    Test {
+        #[clap(
+            help = "Path to the directory containing the cdylib the scripts will be testing against."
+        )]
+        cdylib_dir: OsString,
+
+        #[clap(help = "Path to the UDL file.")]
+        udl_file: OsString,
+
+        #[clap(help = "Foreign language(s) test scripts to run.")]
+        test_scripts: Vec<String>,
+
+        #[clap(
+            long,
+            short,
+            help = "Path to the optional uniffi config file. If not provided, uniffi-bindgen will try to guess it from the UDL's file location."
+        )]
+        config: Option<OsString>,
+    },
+}
+
 pub fn run_main() -> Result<()> {
-    const POSSIBLE_LANGUAGES: &[&str] = &["kotlin", "python", "swift", "ruby"];
-    let matches = clap::App::new("uniffi-bindgen")
-        .about("Scaffolding and bindings generator for Rust")
-        .version(clap::crate_version!())
-        .subcommand(
-            clap::SubCommand::with_name("generate")
-                .about("Generate foreign language bindings")
-                .arg(
-                    clap::Arg::with_name("language")
-                        .required(true)
-                        .takes_value(true)
-                        .long("--language")
-                        .short("-l")
-                        .multiple(true)
-                        .number_of_values(1)
-                        .possible_values(POSSIBLE_LANGUAGES)
-                        .help("Foreign language(s) for which to build bindings"),
-                )
-                .arg(
-                    clap::Arg::with_name("out_dir")
-                        .long("--out-dir")
-                        .short("-o")
-                        .takes_value(true)
-                        .help("Directory in which to write generated files. Default is same folder as .udl file."),
-                )
-                .arg(
-                    clap::Arg::with_name("no_format")
-                        .long("--no-format")
-                        .help("Do not try to format the generated bindings"),
-                )
-                .arg(clap::Arg::with_name("udl_file").required(true))
-                .arg(
-                    clap::Arg::with_name("config")
-                    .long("--config-path")
-                    .takes_value(true)
-                    .help("Path to the optional uniffi config file. If not provided, uniffi-bindgen will try to guess it from the UDL's file location.")
-                ),
-        )
-        .subcommand(
-            clap::SubCommand::with_name("scaffolding")
-                .about("Generate Rust scaffolding code")
-                .arg(
-                    clap::Arg::with_name("out_dir")
-                        .long("--out-dir")
-                        .short("-o")
-                        .takes_value(true)
-                        .help("Directory in which to write generated files. Default is same folder as .udl file."),
-                )
-                .arg(
-                    clap::Arg::with_name("manifest")
-                    .long("--manifest-path")
-                    .takes_value(true)
-                    .help("Path to crate's Cargo.toml. If not provided, Cargo.toml is assumed to be in the UDL's file parent folder.")
-                )
-                .arg(
-                    clap::Arg::with_name("config")
-                    .long("--config-path")
-                    .takes_value(true)
-                    .help("Path to the optional uniffi config file. If not provided, uniffi-bindgen will try to guess it from the UDL's file location.")
-                )
-                .arg(
-                    clap::Arg::with_name("no_format")
-                        .long("--no-format")
-                        .help("Do not format the generated code with rustfmt (useful for maintainers)"),
-                )
-                .arg(clap::Arg::with_name("udl_file").required(true)),
-        )
-        .subcommand(
-            clap::SubCommand::with_name("test")
-            .about("Run test scripts against foreign language bindings")
-            .arg(clap::Arg::with_name("cdylib_dir").required(true).help("Path to the directory containing the cdylib the scripts will be testing against."))
-            .arg(clap::Arg::with_name("udl_file").required(true))
-            .arg(clap::Arg::with_name("test_scripts").required(true).multiple(true).help("Foreign language(s) test scripts to run"))
-            .arg(
-                clap::Arg::with_name("config")
-                .long("--config-path")
-                .takes_value(true)
-                .help("Path to the optional uniffi config file. If not provided, uniffi-bindgen will try to guess from the UDL's file location.")
-            )
-        )
-        .get_matches();
-    match matches.subcommand() {
-        ("generate", Some(m)) => crate::generate_bindings(
-            m.value_of_os("udl_file").unwrap(), // Required
-            m.value_of_os("config"),
-            m.values_of("language").unwrap().collect(), // Required
-            m.value_of_os("out_dir"),
-            !m.is_present("no_format"),
-        )?,
-        ("scaffolding", Some(m)) => crate::generate_component_scaffolding(
-            m.value_of_os("udl_file").unwrap(), // Required
-            m.value_of_os("config"),
-            m.value_of_os("out_dir"),
-            !m.is_present("no_format"),
-        )?,
-        ("test", Some(m)) => {
-            crate::run_tests(
-                m.value_of_os("cdylib_dir").unwrap(), // Required
-                &[&m.value_of_os("udl_file").unwrap().to_string_lossy()], // Required
-                m.values_of("test_scripts").unwrap().collect(), // Required
-                m.value_of_os("config"),
-            )?
-        }
-        _ => bail!("No command specified; try `--help` for some help."),
-    }
+    let cli = Cli::parse();
+    match &cli.command {
+        Commands::Generate {
+            language,
+            out_dir,
+            no_format,
+            config,
+            udl_file,
+        } => crate::generate_bindings(
+            udl_file,
+            config.as_ref(),
+            language.iter().map(String::as_str).collect(),
+            out_dir.as_ref(),
+            !no_format,
+        ),
+        Commands::Scaffolding {
+            out_dir,
+            config,
+            no_format,
+            udl_file,
+        } => crate::generate_component_scaffolding(
+            udl_file,
+            config.as_ref(),
+            out_dir.as_ref(),
+            !no_format,
+        ),
+        Commands::Test {
+            cdylib_dir,
+            udl_file,
+            test_scripts,
+            config,
+        } => crate::run_tests(
+            cdylib_dir,
+            &[&udl_file.to_string_lossy()], // XXX - kinda defeats the purpose of OsString?
+            test_scripts.iter().map(String::as_str).collect(),
+            config.as_ref(),
+        ),
+    }?;
     Ok(())
 }
 
